@@ -1,7 +1,7 @@
-use crate::{msg::Message, MessageType};
+use crate::{crawlers::utils::create_conversion_thread, msg::Message, MessageType};
 use crypto_markets::MarketType;
 use crypto_ws_client::*;
-use std::sync::{Arc, Mutex};
+use std::sync::mpsc::Sender;
 
 use super::crawl_event;
 
@@ -10,24 +10,17 @@ const EXCHANGE_NAME: &str = "kucoin";
 pub(crate) fn crawl_bbo(
     market_type: MarketType,
     symbols: Option<&[String]>,
-    on_msg: Arc<Mutex<dyn FnMut(Message) + 'static + Send>>,
+    tx: Sender<Message>,
     duration: Option<u64>,
 ) {
     if market_type == MarketType::Spot && (symbols.is_none() || symbols.unwrap().is_empty()) {
-        let on_msg_ext = Arc::new(Mutex::new(move |msg: String| {
-            let message = Message::new(
-                EXCHANGE_NAME.to_string(),
-                market_type,
-                MessageType::BBO,
-                msg,
-            );
-            (on_msg.lock().unwrap())(message);
-        }));
+        let tx =
+            create_conversion_thread(EXCHANGE_NAME.to_string(), MessageType::BBO, market_type, tx);
 
         // https://docs.kucoin.com/#all-symbols-ticker
         let channels: Vec<String> = vec!["/market/ticker:all".to_string()];
 
-        let ws_client = KuCoinSpotWSClient::new(on_msg_ext, None);
+        let ws_client = KuCoinSpotWSClient::new(tx, None);
         ws_client.subscribe(&channels);
         ws_client.run(duration);
     } else {
@@ -36,7 +29,7 @@ pub(crate) fn crawl_bbo(
             MessageType::BBO,
             market_type,
             symbols,
-            on_msg,
+            tx,
             duration,
         );
     }
