@@ -75,6 +75,7 @@ fn get_lock_file_name(exchange: &str, market_type: MarketType, prefix: &str) ->
                     | MarketType::LinearSwap
                     | MarketType::InverseFuture => "kucoin_swap.lock".to_string(),
                     MarketType::Spot => "kucoin_spot.lock".to_string(),
+                    MarketType::Unknown => "kucoin_unknown.lock".to_string(), // for OpenInterest
                     _ => panic!("Unknown market_type {} of {}", market_type, exchange),
                 }
             }
@@ -123,7 +124,7 @@ fn create_all_lock_files(
             }
         } else {
             if *exchange == "ftx" || *exchange == "kucoin" {
-                market_types.push(MarketType::Unknown);
+                market_types.push(MarketType::Unknown); // for OpenInterest
             }
         }
         for market_type in market_types {
